/*
 * Polynomial.java
 *
 * Created on 27 mai 2009, 15:52
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package assignment2;
import java.util.Random;
/**
 *
 * @author Aqualonne
 */
public class Polynomial {
    private Node head;  // only a head node
    
    /** Creates a new instance of Polynomial */
    public Polynomial() {
        head = null;    // empty means nothing to start with
    }
    
    public double estimateRoot(int a, int b, double c){
        double current = a+c;
        int lastValue = evaluate(a);
        while(current<=b){
            int currentValue = evaluate((int)current);
            if( (currentValue<0 && lastValue>0) || (currentValue > 0 && lastValue<0) ){
                //System.out.println("P(" + current + ") = " + currentValue + ", compared to " + lastValue);
                return current;
            }
            current += c;
            lastValue = currentValue;
        }
        return 0.0;
    }
    
    public static Polynomial generates(int highestDegree, int highestConstant){
        Random r = new Random();
        Polynomial p = new Polynomial();
        for(int i = 0; i<=highestDegree; i++){
            int currentConstant = r.nextInt(highestConstant);
            if(currentConstant != 0)    // IMPORTANT NO TO ADD EMPTY MONOMIALS
                p.add(currentConstant,i);
        }
        return p;
    }
    
    // for multiply
    public boolean isEmpty(){ return head==null; }
    
    public Polynomial multiply(Polynomial p1, Polynomial p2){
        Polynomial result = new Polynomial();
        if(p1.isEmpty() || p2.isEmpty()) return result; //multiply by 0, get 0
        Node node1 = p1.head;   // it's private but from a Polynomial, so we can access
        Node node2 = p2.head;
        while(node1!=null){
            while(node2!=null){
                result.add(node1.constant*node2.constant,node1.exponent+node2.exponent);
                node2 = node2.next;
            }
            node1 = node1.next;
            node2 = p2.head;
        }
        return result;
    }
    
    public int evaluate(int x){
        Node n = head;
        int result = 0;
        while(n != null){
            result += n.constant * Math.pow(x,n.exponent);
            n = n.next;
        }
        return result;
    }
    
    public void add(int constant, int exponent){
        Node current = get(exponent);
        if(current!=null){//already one with same exponent
            current.constant += constant;
            //if we are left with 0, then we should take it out
            remove(exponent);//not efficient since we're doing two look-up
        }else{//add a new one
            current = new Node(constant,exponent);
            current.next = head;
            head = current;
            //we were not asked to keep any order!
        }
    }
    
    /**
     * Removes the node with the given exponent.
     *
     * @param exponent The exponent of the node to be removed.
     */
    private void remove(int exponent){
        if(head.exponent==exponent){
            head = head.next;
            return;
        }
        Node current = head;
        while(current.next.exponent!=exponent)
            current = current.next;
        current.next = current.next.next;
    }
    
    /**
     * Return a node with the given exponent, or null if there is none.
     *
     * @param exponent Exponent of the node we want
     * @return Node with the exponent, or null if none is found.
     */
    private Node get(int exponent){
        Node n = head;
        while(n!=null && n.exponent!=exponent)
            n = n.next;
        return n;
    }
    
    public String toString(){
        String s = "";
        Node n = head;
        while(n!=null){
            s += n.toString();
            n = n.next;
        }
        return s;
    }
    
    private class Node{ // a monomial
        public Node next;
        public int constant;
        public int exponent;
        public Node(int c, int e){
            constant = c;
            exponent = e;
            next = null;
        }
        public String toString(){
            return constant+"x^"+exponent + " ";
        }
    }
}
